/**
 * @file rgbx.h
 *
 * @author David Betz
 *
 * @version 0.01
 *
 * @copyright
 * Copyright (c) David Betz 2014, All Rights MIT Licensed.
 *
 * @brief Driver for RGBx LEDs.
 */

#ifndef __rgbx_H__
#define __rgbx_H__

#include <stdint.h>

#if defined(__cplusplus)
extern "C" {
#endif

#define TYPE_RGB            0
#define TYPE_GRB            1   // for WS2812+

#define COLOR(r, g, b, w)      (((r) <<24) | ((g) << 16)| ((b) << 8) | (w))
#define SCALE(x, l)         ((x) * (l) / 255)
#define COLORX(r, g, b, w, l)  ((SCALE(r, l) << 24) | (SCALE(g, l) << 16) | SCALE(b, l) << 8| SCALE(b, l))

//                         RRGGBBWW
#define COLOR_BLACK      0x00000000
#define COLOR_RED        0xFF000000
#define COLOR_GREEN      0x00FF0000
#define COLOR_BLUE       0x0000FF00
#define COLOR_WHITE      0xFFFFFF00
#define COLOR_CYAN       0x00FFFF00
#define COLOR_MAGENTA    0xFF00FF00
#define COLOR_YELLOW     0xFFFF0000
#define COLOR_CHARTREUSE 0x7FFF0000
#define COLOR_ORANGE     0xFF600000
#define COLOR_AQUAMARINE 0x7FFFD400
#define COLOR_PINK       0xFF5F5F00
#define COLOR_TURQUOISE  0x3FE0C000
#define COLOR_REALWHITE  0xC8FFFF00
#define COLOR_INDIGO     0x3F007F00
#define COLOR_VIOLET     0xBF7FBF00
#define COLOR_MAROON     0x32001000
#define COLOR_BROWN      0x0E060000
#define COLOR_CRIMSON    0xDC283C00
#define COLOR_PURPLE     0x8C00FF00

// driver state structure
typedef struct {
    volatile uint32_t command;
    int cog;
} rgbx_t;

// simpler type name for use with SimpleIDE
typedef rgbx_t rgbx;

/**
 * @brief Start a driver for a chain of sk6812RGBW LEDs
 *
 * @param pin Pin connected to the first LED
 * @param count Number of LEDs in the chain
 * @returns Driver COG number or -1 on failure
 *
 */
int sk6812RGBW_start(int pin, int count);

/**
 * @brief Start a driver for a chain of sk6812 LEDs
 *
 * @param pin Pin connected to the first LED
 * @param count Number of LEDs in the chain
 * @returns Driver COG number or -1 on failure
 *
 */
int sk6812_start(int pin, int count);
/**
 * @brief Start a driver for a chain of ws2811 LEDs
 *
 * @param pin Pin connected to the first LED
 * @param count Number of LEDs in the chain
 * @returns Driver COG number or -1 on failure
 *
 */

int ws2811_start(int pin, int count);

/**
 * @brief Start a driver for a chain of WS2812 LEDs
 *
 * @param pin Pin connected to the first LED
 * @param count Number of LEDs in the chain
 * @returns Driver COG number or -1 on failure
 *
 */
int ws2812_start(int pin, int count);

/**
 * @brief Start a driver for a chain of WS2812b LEDs
 *
 * @param pin Pin connected to the first LED
 * @param count Number of LEDs in the chain
 * @returns Driver COG number or -1 on failure
 *
 */
int ws2812b_start(int pin, int count);

/**
 * @brief Set the colors to display
 *
 * @param colors Array of colors, one for each LED in the chain
 */
void rgbx_set(int *colors);

/**
 * @brief Open a driver for sk6812RGBW chips
 * 
 * @returns A pointer to the driver structure or NULL on failure
 */
rgbx_t *sk6812RGBW_open(void);

/**
 * @brief Open a driver for sk6812 chips
 * 
 * @returns A pointer to the driver structure or NULL on failure
 */
rgbx_t *sk6812_open(void);
         
/**
 * @brief Open a driver for WS2811 chips
 * 
 * @returns A pointer to the driver structure or NULL on failure
 */
rgbx_t *ws2811_open(void);        
         
/**
 * @brief Open a driver for WS2812 chips
 * 
 * @returns A pointer to the driver structure or NULL on failure
 */
rgbx_t *ws2812_open(void);

/**
 * @brief Open a driver for WS2812B chips
 * 
 * @returns A pointer to the driver structure or NULL on failure
 */
rgbx_t *ws2812b_open(void);

/**
 * @brief Close a RGBX driver
 *
 * @param driver Pointer to the driver structure
 */
void rgbx_close(rgbx_t *driver);

/**
 * @brief Initialize a driver for RGBx chips
 *
 * @param driver Pointer to a driver structure
 * @returns Driver COG number or -1 on failure
 */
int ws2812_init(rgbx_t *driver);

/**
 * @brief Initialize a driver for RGBx chips
 *
 * @param driver Pointer to a driver structure
 * @returns Driver COG number or -1 on failure
 */
int ws2812b_init(rgbx_t *driver);

/**
 * @brief Load a COG with a driver using custom parameters
 *
 * @param usreset Reset timing (us)
 * @param ns0h 0-bit high timing (ns)
 * @param ns0l 0-bit low timing (ns)
 * @param ns1h 1-bit high timing (ns)
 * @param ns1l 1-bit low timing (ns)
 * @param type color format (either TYPE_RGB or TYPE_GRB)
 * @param bits to shift pixels 
 * @returns Driver COG number or -1 on failure
 */
int ws_init(rgbx_t *driver, int usreset, int ns0h, int ns0l, int ns1h, int ns1l, int type, int bits);

/**
 * @brief Shut down the COG running a driver
 *
 * @param driver Pointer to the driver structure
 */
void rgbx_term(rgbx_t *driver);

/**
 * @brief Update a chain of LEDs
 *
 * @param driver Pointer to the driver structure
 * @param pin Pin connected to the first LED
 * @param colors Array of colors, one for each LED in the chain
 * @param count Number of LEDs in the chain
 */
void rgbx_update(rgbx_t *driver, int pin, uint32_t *colors, int count);

/**
 * @brief Create color from a 0 to 255 position input
 *
 * @detail Colors transition red to green to blue.
 *
 * @param pos Position in the color spectrum where 0 is on the red end and 255 is on the blue end
 * @returns Color at the specified position
 */
uint32_t rgbx_wheel(int pos);

/**
 * @brief Create color from a 0 to 255 position input
 *
 * @detail Colors transition red to green to blue.
 *
 * @param pos Position in the color spectrum where 0 is on the red end and 255 is on the blue end
 * @param brightness The brightness of the generated color where 0 is off and 255 is full
 * @returns Color at the specified position
 */
uint32_t rgbx_wheel_dim(int pos, int brightness);

#if defined(__cplusplus)
}
#endif

#endif

/**
 * TERMS OF USE: MIT License
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
